@@ -1,28 +1,31 @@
 /*
- * src/sm_drv_spi.c
+ * This file is part of cx3110x
  *
  * Copyright (C) 2003 Conexant Americas Inc. All Rights Reserved.
  * Copyright (C) 2004, 2005, 2006 Nokia Corporation
- * Author: Samuel Ortiz <samuel.ortiz@nokia.com>
  *
+ * Contact: Kalle Valo <Kalle.Valo@nokia.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License 
+ * version 2 as published by the Free Software Foundation. 
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  *
  */
+
 #define __KERNEL_SYSCALLS__
 
 #include <linux/version.h>
-#ifdef MODULE
-#ifdef MODVERSIONS
-#include <linux/modversions.h>
-#endif
 #include <linux/module.h>
-#else
-#define MOD_INC_USE_COUNT
-#define MOD_DEC_USE_COUNT
-#endif
 
 #include <linux/init.h>
 #include <linux/netdevice.h>
@@ -48,16 +51,11 @@
 #include "sm_drv_pda.h"
 #include "pda.h"
 
-static int sm_drv_spi_up(struct net_device *dev);
-static int sm_drv_spi_down(struct net_device *dev);
-static void sm_drv_spi_cli(struct net_device *dev);
 static void sm_drv_spi_initialize(struct net_device *dev);
 static void sm_drv_spi_reset_device(struct net_device *dev, unsigned boot);
-static irqreturn_t sm_drv_spi_interrupt(int irq, void *config, struct pt_regs *regs);
+static irqreturn_t sm_drv_spi_interrupt(int irq, void *config);
 static struct net_device *sm_drv_spi_probe(void);
 
-#define DRIVER_NAME     "CX3110x"
-#define VERSIONID       "0.8"
 const char driver_name[64] = DRIVER_NAME;
 char wlan_fw_version[32] = "N/A";
 char * firmware_file;
@@ -68,7 +66,8 @@ extern struct net_device *sm_net_device;
 
 extern unsigned int driver_type;
 
-static u8 wlan_wait_bit(struct net_device * dev, u16 reg, u32 mask, u32 expected )
+static u8 wlan_wait_bit(struct net_device * dev, u16 reg, u32 mask,
+			u32 expected )
 {
 	int i;
 	char buffer[4];
@@ -84,12 +83,9 @@ static u8 wlan_wait_bit(struct net_device * dev, u16 reg, u32 mask, u32 expected
 	return 0;
 }
 
-/******************************************************************************
- *   SPI Interconnect Handling (Bottom halve and helpers)
- ******************************************************************************/
-
 struct s_txdata {
-	struct s_ic_msg *ic_msg;                    /* ic message associated with packet */
+	/* ic message associated with packet */
+	struct s_ic_msg *ic_msg;                   
 	uint32_t ic_msg_flags;
 };
 
@@ -105,15 +101,23 @@ static void sm_drv_send_zero_mac(struct net_device *dev)
 	wireless_send_event(dev, SIOCGIWAP, &uwrq, NULL);
 }
 
-#ifdef CONFIG_MACH_NOKIA_N800
 static int sm_drv_spi_wakeup(struct net_device *dev) 
 {
 	struct net_local *lp = dev->priv;
 	uint32_t host_ints, host_ints_ack, target_ints;
 	unsigned long timeout;
-	int err;
-	
+	int err, tries = 0;
+
+again:	
 	DEBUG(DBG_BH, "w\n");
+
+	spin_lock_bh(&lp->lock);
+	if (lp->device_state != DEVSTATE_ACTIVE) {
+		spin_unlock_bh(&lp->lock);
+		err = -1;
+		goto exit;
+	}
+	spin_unlock_bh(&lp->lock);
 	
 	/* Here we wake the target up */
 	target_ints = SPI_TARGET_INT_WAKEUP; 
@@ -121,20 +125,36 @@ static int sm_drv_spi_wakeup(struct net_device *dev)
 		     (unsigned char *)&target_ints, sizeof target_ints );
 		
 	/* And wait for the READY interrupt */
-	timeout = jiffies + 2 * HZ;
+	timeout = jiffies + HZ;
+	
 	sm_spi_read(dev, SPI_ADRS_HOST_INTERRUPTS,
 		    (unsigned char *)&host_ints, sizeof(host_ints));
 	while (!(host_ints & SPI_HOST_INT_READY)) {
 		DEBUG(DBG_BH, ".\n");
 		if (time_after(jiffies, timeout)) {
-			printk(KERN_WARNING
-			       "cx3110x: We haven't got a READY interrupt"
-			       " from WAKEUP. (firmware crashed?)\n");
-			lp->device_state = DEVSTATE_DEAD;
-			err = -1;
-			goto exit;
-			
+			if (tries < 3) {
+				tries++;
+				cx3110x_warning("Haven't got a READY interrupt"
+						" from WAKEUP, trying again. "
+						"(host_ints=0x%x, tries=%i)",
+						host_ints, tries);
+				goto again;
+			} else {
+				cx3110x_error("Chip did not respond to "
+					      "WAKEUP, it's dead. "
+					      " Firmware crashed? "
+					      "(host_ints=0x%x, tries=%i)",
+					      host_ints, tries);
+				spin_lock_bh(&lp->lock);
+				lp->device_state = DEVSTATE_DEAD;
+				spin_unlock_bh(&lp->lock);
+				err = -1;
+				goto exit;
+			}
 		}
+		
+		msleep(1);
+		
 		sm_spi_read(dev, SPI_ADRS_HOST_INTERRUPTS,
 			    (unsigned char *)&host_ints, sizeof(host_ints));
 	}
@@ -144,6 +164,10 @@ static int sm_drv_spi_wakeup(struct net_device *dev)
 		     (unsigned char *)&host_ints_ack, sizeof(host_ints_ack));
 
 	err = 0;
+
+	if (tries > 0)
+		cx3110x_info("chip woke up (host_ints=0x%x, tries=%i)",
+			     host_ints, tries);
 	
 exit:
 	DEBUG(DBG_BH, "W\n");
@@ -170,7 +194,7 @@ static void sm_drv_spi_reset(struct net_device *dev)
 	uint32_t target_ints = SPI_TARGET_INT_RDDONE;
 	uint32_t bgr_scan_disable = 0;
 
-	printk("SCAN timer expired, resetting WLAN\n");
+	cx3110x_warning("scan timer expired, resetting.");
 
 	sm_spi_write(dev, SPI_ADRS_ARM_INTERRUPTS,
 		     (unsigned char *)&target_ints, sizeof(target_ints));
@@ -178,22 +202,27 @@ static void sm_drv_spi_reset(struct net_device *dev)
 	wrqu.data.length = 0;
 	wrqu.data.flags = 0;
 
-	if (lp->bss_type != DOT11_BSSTYPE_IBSS)
+	spin_lock_bh(&lp->lock);
+
+	if (lp->bss_type != DOT11_BSSTYPE_IBSS) {
+		spin_unlock_bh(&lp->lock);
 		sm_drv_oid_set(dev, DOT11_OID_AUTOSCANDISABLE,
 			       (void*)&bgr_scan_disable,
 			       sizeof(uint32_t));
+		spin_lock_bh(&lp->lock);
+	}
 
 	wireless_send_event(dev, SIOCGIWSCAN, &wrqu, NULL);
 	sm_drv_send_zero_mac(dev);
-	spin_lock_bh(&lp->sm_lock);
 	lp->device_state = DEVSTATE_DEAD;
-	spin_unlock_bh(&lp->sm_lock);
+
+	spin_unlock_bh(&lp->lock);
 }
 
 static int sm_drv_spi_rx(struct net_device *dev) 
 {
 	struct net_local *lp = dev->priv;
-	struct spi_hif_local_data *hif_lp = HIF_LP(lp);
+	struct spi_local *spi_lp = lp->spi_lp;
 	struct s_sm_frame *frame;
 	struct s_ic_msg *ic_msg;
 	unsigned short length;
@@ -205,8 +234,12 @@ static int sm_drv_spi_rx(struct net_device *dev)
 		goto exit;
 	}
 
+	spin_lock_bh(&lp->lock);
+
 	frame = frame_skb_alloc(dev, lp->sm_descr.mtu + lp->sm_descr.rxoffset,
 				0);
+
+	spin_unlock_bh(&lp->lock);
 	
 	if (frame != NULL) {
 		ic_msg = FRAME_ICMSG(frame);
@@ -220,9 +253,8 @@ static int sm_drv_spi_rx(struct net_device *dev)
 			    (unsigned char *)&length, sizeof(length));
 
 		if (length == 0) {
-			printk(KERN_WARNING "cx3110x: WARNING"
-			       " chip requested DMA rx transfer"
-			       " a zero length frame\n");
+			cx3110x_warning("chip requested DMA rx transfer"
+					" of a zero length frame");
 			frame_skb_free(dev, frame);
 			err = -1;
 			goto exit;
@@ -242,13 +274,16 @@ static int sm_drv_spi_rx(struct net_device *dev)
 		ic_msg->length  = length;
 		ic_msg->address = 0;
 		ic_msg->data    = frame->data;
+
+		spin_lock_bh(&lp->lock);
 			
-		hif_lp->spi_packets++;
-		spin_lock_bh(&lp->sm_lock);
+		spi_lp->spi_packets++;
+
 		callb_mask = prism_interconnect_message_handle(lp->sm_context,
 							       ic_msg);
-		spin_unlock_bh(&lp->sm_lock);
-			
+
+		spin_unlock_bh(&lp->lock);
+
 		DEBUG(DBG_IC, "After prism_interconnect_message_handle"
 		      "(%d, %x, %d, %x, %p)\n",
 		      ic_msg->channel, ic_msg->flags, ic_msg->length,
@@ -257,11 +292,12 @@ static int sm_drv_spi_rx(struct net_device *dev)
 		DEBUG(DBG_IC,"Callback mask: %d\n", callb_mask);
 			
 		if(callb_mask < 0)
-			printk(KERN_WARNING "prism_interconnect_message_handle"
-			       " returned error %d\n", callb_mask);
+			cx3110x_warning("prism_interconnect_message_handle"
+					" returned error %d",
+					callb_mask);
 	} else
-		printk("Couldn't allocate RX frame\n");
-	
+		cx3110x_warning("couldn't allocate RX frame.");
+
 	handle_sm_callback(dev, callb_mask);
 
 exit:
@@ -271,7 +307,7 @@ exit:
 static int sm_drv_spi_tx(struct net_device *dev) 
 {
 	struct net_local *lp = dev->priv;
-	struct spi_hif_local_data *hif_lp = HIF_LP(lp);
+	struct spi_local *spi_lp = lp->spi_lp;
 	struct s_ic_msg *ic_msg;
 	struct s_dma_regs dma_regs;
 	int32_t callb_mask = 0;
@@ -284,11 +320,11 @@ static int sm_drv_spi_tx(struct net_device *dev)
 	{
 
 		ic_mask = IC_MASK_CTRL | IC_MASK_FRAME;
-		spin_lock_bh(&lp->sm_lock);
+		spin_lock_bh(&lp->lock);
 		callb_mask = prism_interconnect_message_query(lp->sm_context,
 							      ic_mask,
 							      &ic_msg);
-		spin_unlock_bh(&lp->sm_lock);
+		spin_unlock_bh(&lp->lock);
 		if(!ic_msg) {
 			err = 0;
 			goto exit;
@@ -301,7 +337,7 @@ static int sm_drv_spi_tx(struct net_device *dev)
 
 		/* Firmware bug...First TX packets needs to be horribly
 		 * delayed */
-		if (hif_lp->initial_packets < SPI_DELAY_THRESHOLD)
+		if (spi_lp->initial_packets < SPI_DELAY_THRESHOLD)
 			msleep(5);
 
 		err = sm_drv_spi_wakeup(dev);
@@ -333,9 +369,9 @@ static int sm_drv_spi_tx(struct net_device *dev)
 		while(!(host_ints & SPI_HOST_INT_WR_READY)) {
 			DEBUG(DBG_BH, "-\n");
 			if (time_after(jiffies, timeout)) {
-				printk(KERN_WARNING "cx3110x: Haven't got a "
-				       "WR_READY for DMA write."
-				       "(firmware crashed?)\n");
+				cx3110x_warning("Haven't got a "
+						"WR_READY for DMA write. "
+						"(firmware crashed?)");
 				err = -1;
 				goto exit;
 			}
@@ -352,11 +388,15 @@ static int sm_drv_spi_tx(struct net_device *dev)
 		DEBUG(DBG_BH, "WR\n");
 
 		if (ic_msg->flags & IC_FLAG_FREE) {
+			spin_lock_bh(&lp->lock);
+
 			if (ic_msg->frame)
 				prism_driver_frame_free(lp->sm_context,
 							ic_msg->frame);
 			else
 				prism_driver_free(lp->sm_context, ic_msg);
+
+			spin_unlock_bh(&lp->lock);
 		}
 	}
 
@@ -366,16 +406,14 @@ exit:
 	return err;
 }
 
-static void sm_drv_spi_wq(void * data)
+static void sm_drv_spi_wq(struct work_struct *work)
 {
-	struct net_device *dev = (struct net_device *)data;
-	struct net_local *lp = dev->priv;
-	struct spi_hif_local_data *hif_lp = HIF_LP(lp);
+	struct net_local *lp = container_of(work, struct net_local, work);
+	struct net_device *dev = lp->netdev;
+	struct spi_local *spi_lp = lp->spi_lp;
 	uint32_t host_ints, host_ints_en, host_ints_ack;
 	int err;
 	
-	HIF_CLK_USE(hif_lp);
-
 	sm_spi_read(dev, SPI_ADRS_HOST_INTERRUPTS,
 		    (unsigned char *) &host_ints, sizeof(host_ints));
 
@@ -386,9 +424,16 @@ static void sm_drv_spi_wq(void * data)
 		sm_spi_write(dev, SPI_ADRS_HOST_INT_ACK,
 			     (unsigned char *)&host_ints_ack,
 			     sizeof(host_ints_ack));
-		if (hif_lp->upload_state == UPLOAD_STATE_BOOTING) {
+
+		spin_lock_bh(&lp->lock);
+		if (spi_lp->upload_state == UPLOAD_STATE_BOOTING) {
+			spin_unlock_bh(&lp->lock);
+
 			sm_drv_spi_initialize(dev);
-			hif_lp->upload_state = UPLOAD_STATE_RUNNING;
+
+			spin_lock_bh(&lp->lock);
+			spi_lp->upload_state = UPLOAD_STATE_RUNNING;
+			spin_unlock_bh(&lp->lock);
 
 			host_ints_en = SPI_HOST_INT_UPDATE |
 				SPI_HOST_INT_SW_UPDATE;
@@ -400,18 +445,29 @@ static void sm_drv_spi_wq(void * data)
 
 			/* Now we can wait ifconfig...*/
 			complete(&softmac_init_comp);
-		}		
+
+			spin_lock_bh(&lp->lock);
+		}
+		
+		spin_unlock_bh(&lp->lock);
+
 	}
 
+	spin_lock_bh(&lp->lock);
+
 	if (unlikely(lp->device_state == DEVSTATE_RESETTING)) {
+		spin_unlock_bh(&lp->lock);
 		sm_drv_spi_reset(dev);
 		goto exit;
 	}
 
 	if (lp->sm_descr.mtu == 0) {
+		spin_unlock_bh(&lp->lock);
 		DEBUG(DBG_BH, "%s SoftMAC not initialized\n", DRIVER_NAME);
 		goto exit;
 	}
+
+	spin_unlock_bh(&lp->lock);
     
 	/* Check if LMAC has rx frame */
 	if((host_ints & SPI_HOST_INT_UPDATE) ||
@@ -432,185 +488,12 @@ static void sm_drv_spi_wq(void * data)
 		goto exit;
 	}
 
-	/* The target can go back to sleep again */
-	sm_drv_spi_sleep(dev);
-	
 exit:
-	HIF_CLK_UNUSE(hif_lp);
+	sm_drv_spi_sleep(dev);	
+
+	return;
 }
-#endif /* CONFIG_MACH_NOKIA_N800 */
 
-#ifdef CONFIG_MACH_NOKIA770
-static void sm_drv_spi_tasklet_action(unsigned long data)
-{
-	struct net_device *dev = (struct net_device *)data;
-	struct net_local *lp = dev->priv;
-	struct spi_hif_local_data *hif_lp = HIF_LP(lp);
-	struct s_sm_frame *frame;
-	struct s_ic_msg *ic_msg;
-	struct s_txdata txdata;
-	uint32_t ic_mask = 0;
-	int32_t callb_mask = 0;
-	struct s_dma_regs dma_regs;
-	uint32_t host_ints, host_int_en, target_ints;
-	unsigned short length;
-
-	HIF_CLK_USE(hif_lp);
-
-	/* The IRQ line won't be updated, but the register still will be */
-	host_int_en = 0;
-	sm_spi_write(dev, SPI_ADRS_HOST_INT_EN, (unsigned char *)&host_int_en, sizeof(host_int_en));
-
-	sm_spi_read(dev, SPI_ADRS_HOST_INTERRUPTS, (unsigned char *)&host_ints, sizeof(host_ints));
-
-	DEBUG(DBG_BH, "%s interrupts 0x%x\n", DRIVER_NAME, host_ints);
-
-	host_ints &= SPI_HOST_INTS_DEFAULT;
-	sm_spi_write(dev, SPI_ADRS_HOST_INT_ACK, (unsigned char *)&host_ints, sizeof(host_ints));
-
-	if(host_ints & SPI_HOST_INT_READY) {
-		if (hif_lp->upload_state == UPLOAD_STATE_BOOTING) {
-			sm_drv_spi_initialize(dev);
-			hif_lp->upload_state = UPLOAD_STATE_RUNNING;
-			/* Now we can wait ifconfig...*/
-			complete(&softmac_init_comp);
-		}		
-	}
-
-	if (lp->sm_descr.mtu == 0) {
-		DEBUG(DBG_BH, "%s SoftMAC not initialized\n", DRIVER_NAME);
-		HIF_CLK_UNUSE(hif_lp);
-		return;
-	}
-    
-	/* Check if LMAC has rx frame */
-	if((host_ints & SPI_HOST_INT_UPDATE) || (host_ints & SPI_HOST_INT_SW_UPDATE)) {
-		
-		if ((frame = frame_skb_alloc(dev, lp->sm_descr.mtu + lp->sm_descr.rxoffset, 0)) != NULL) {
-        
-			ic_msg = FRAME_ICMSG(frame);
-			ic_msg->frame = frame;
-            
-			/* dummy read to flush SPI DMA controller bug */
-			sm_spi_read(dev, SPI_ADRS_GEN_PURP_1, (unsigned char *)&length, sizeof(length));
-
-			sm_spi_read(dev, SPI_ADRS_DMA_DATA, (unsigned char *)&length, sizeof(length));
-            
-			DEBUG(DBG_BH, "%s received frame len=%d\n", DRIVER_NAME, length);
-            
-			if ( length > SPI_MAX_PACKET_SIZE ) 
-				length = SPI_MAX_PACKET_SIZE;
-
-			sm_spi_dma_read(dev, SPI_ADRS_DMA_DATA, (unsigned char *)frame->data, length);
-			
-			ic_msg->channel = 0; 
-			ic_msg->flags   = 0;
-			ic_msg->length  = length;
-			ic_msg->address = 0;
-			ic_msg->data    = frame->data;
-			
-			hif_lp->spi_packets++;
-			spin_lock_bh(&lp->sm_lock);
-			callb_mask = prism_interconnect_message_handle(lp->sm_context, ic_msg);
-			spin_unlock_bh(&lp->sm_lock);
-			
-			DEBUG(DBG_IC, "After prism_interconnect_message_handle(%d, %x, %d, %x, %p)\n",
-			      ic_msg->channel, ic_msg->flags, ic_msg->length,
-			      ic_msg->address, ic_msg->data);
-			
-			DEBUG(DBG_IC,"Callback mask: %d\n", callb_mask);
-			
-			if(callb_mask < 0)
-				printk(KERN_WARNING "prism_interconnect_message_handle returned error %d\n", callb_mask);
-		} else
-			printk("Couldn't allocate RX frame\n");
-	}
-        
-	handle_sm_callback(dev, callb_mask);
-
-	/* See if the UMAC has ic messages pending for us to send */
-	while(1)
-	{
-		unsigned long timeout;
-
-		ic_mask = IC_MASK_CTRL | IC_MASK_FRAME;
-		spin_lock_bh(&lp->sm_lock);
-		callb_mask = prism_interconnect_message_query(lp->sm_context, ic_mask, &ic_msg);
-		spin_unlock_bh(&lp->sm_lock);
-		if(!ic_msg)
-			break;
-
-		DEBUG(DBG_IC, "prism_interconnect_message_query(%d, %x, %d, %x, %p)\n",
-		      ic_msg->channel, ic_msg->flags, ic_msg->length, ic_msg->address, ic_msg->data );
-
-		/* Firmware bug...First TX packets needs to be horribly delayed */
-		if (hif_lp->initial_packets < SPI_DELAY_THRESHOLD)
-			mdelay(5);
-
-		/* Here we wake the target up */
-		target_ints = SPI_TARGET_INT_WAKEUP; 
-		sm_spi_write(dev, SPI_ADRS_ARM_INTERRUPTS, (unsigned char *)&target_ints, sizeof target_ints );
-		
-		/* And wait for the READY interrupt */
-		timeout = jiffies + 2 * HZ;
-		sm_spi_read(dev, SPI_ADRS_HOST_INTERRUPTS, (unsigned char *)&host_ints, sizeof(host_ints));
-		while (!(host_ints & SPI_HOST_INT_READY)) {
-			if (time_after(jiffies, timeout)) {
-				printk("We haven't got a READY interrupt from WAKEUP\n");
-				lp->device_state = DEVSTATE_DEAD;
-				return;
-			}
-			sm_spi_read(dev, SPI_ADRS_HOST_INTERRUPTS, (unsigned char *)&host_ints, sizeof(host_ints));
-		}
-
-		txdata.ic_msg_flags = ic_msg->flags;
-		txdata.ic_msg = ic_msg;
-
-		/* program DMA and transfer TX buffer */
-		dma_regs.cmd  = SPI_DMA_WRITE_CTRL_ENABLE;        
-		dma_regs.len  = cpu_to_le16(ic_msg->length);      
-		dma_regs.addr = cpu_to_le32(ic_msg->address); 
-	
-		sm_spi_write(dev, SPI_ADRS_DMA_WRITE_CTRL, (unsigned char *)&dma_regs, sizeof(struct s_dma_regs));
-		
-		sm_spi_dma_write(dev, SPI_ADRS_DMA_DATA, ic_msg->data, ic_msg->length);
-		
-		/* We wait for the DMA Ready interrupt */
-		host_ints = 0;
-		sm_spi_read(dev, SPI_ADRS_HOST_INTERRUPTS, (unsigned char *)&host_ints, sizeof(host_ints));
-		timeout = jiffies + 2 * HZ;
-		while(!(host_ints & (SPI_HOST_INT_WR_READY | SPI_HOST_INT_READY))) {
-			if (time_after(jiffies, timeout)) {
-				printk("We haven't got a READY interrupt from DMA READY\n");
-				break;
-			}
-			sm_spi_read(dev, SPI_ADRS_HOST_INTERRUPTS, (unsigned char *)&host_ints, sizeof(host_ints));
-		}
-	
-		if (txdata.ic_msg_flags & IC_FLAG_FREE) {
-			if (txdata.ic_msg->frame)
-				prism_driver_frame_free(lp->sm_context, txdata.ic_msg->frame);
-			else
-				prism_driver_free(lp->sm_context, txdata.ic_msg);		
-		}
-	}
-
-	/* The target can go back to sleep again */
-	target_ints = SPI_TARGET_INT_SLEEP; 
-	sm_spi_write(dev, SPI_ADRS_ARM_INTERRUPTS, (unsigned char *)&target_ints, sizeof target_ints );
-
-	handle_sm_callback(dev, callb_mask);
-	
-	host_int_en = SPI_HOST_INTS_DEFAULT;
-	sm_spi_write(dev, SPI_ADRS_HOST_INT_EN, (unsigned char *)&host_int_en, sizeof(host_int_en));
-
-	HIF_CLK_UNUSE(hif_lp);
-}
-#endif /* CONFIG_MACH_NOKIA770 */
-
-/******************************************************************************
- *   Initialization and Firmware Upload. PDA loading too...
- ******************************************************************************/
 static void wlan_omap_device_release(struct device * dev)
 {
 	
@@ -631,16 +514,24 @@ static int wlan_omap_suspend(struct device *dev, pm_message_t state)
 	struct net_device * net_dev;
 	struct net_local *lp;
 
-	DEBUG(DBG_ALL, "WLAN suspending...\n");
+	cx3110x_info("suspending.");
 
 	net_dev = (struct net_device *)dev_get_drvdata(dev);
 	lp = net_dev->priv;
 
-	if (lp->sm_initialization)
+	spin_lock_bh(&lp->lock);
+
+	if (lp->sm_initialization) {
+		spin_unlock_bh(&lp->lock);
 		ret = sm_drv_oid_set((struct net_device *)dev_get_drvdata(dev),
 				     GEN_OID_HIBERNATE,
 				     &hibernate,
 				     sizeof(uint32_t));
+		spin_lock_bh(&lp->lock);
+	}
+
+	spin_unlock_bh(&lp->lock);
+
 	return ret;
 }
 
@@ -651,16 +542,23 @@ static int wlan_omap_resume(struct device *dev)
 	struct net_device * net_dev;
 	struct net_local *lp;
 
-	DEBUG(DBG_ALL, "WLAN resuming...\n");
+	cx3110x_info("resuming.");
 
 	net_dev = (struct net_device *)dev_get_drvdata(dev);
 	lp = net_dev->priv;
 
-	if (lp->sm_initialization)
+	spin_lock_bh(&lp->lock);
+
+	if (lp->sm_initialization) {
+		spin_unlock_bh(&lp->lock);
 		ret = sm_drv_oid_set((struct net_device *)dev_get_drvdata(dev),
 				     GEN_OID_HIBERNATE,
 				     &hibernate,
 				     sizeof(uint32_t));
+		spin_lock_bh(&lp->lock);
+	}
+
+	spin_unlock_bh(&lp->lock);
 
 	return ret;
 }
@@ -692,7 +590,7 @@ int sm_drv_register_platform(struct net_device * sm_drv)
 
 	ret = platform_device_register(&wlan_omap_device);
 	if (ret) {
-		printk("Couldn't register wlan_omap device\n");
+		cx3110x_error("Couldn't register wlan_omap device.");
 		return ret;
 	}
 
@@ -709,50 +607,37 @@ void sm_drv_unregister_platform(void)
 }
 
 
-static int sm_drv_spi_up(struct net_device *dev)
+int sm_drv_spi_up(struct net_device *dev)
 {
 	DEBUG(DBG_CALLS, "%s\n", __FUNCTION__);
 	
-	DEBUG(DBG_ALL, "Loading %s firmware\n", firmware_file);
+	cx3110x_info("loading %s firmware.", firmware_file);
 		
 	if (sm_drv_spi_upload_firmware(dev) < 0) {
-		DEBUG(DBG_ALL, "%s failed\n", firmware_file);
+		cx3110x_error("failed to load %s.", firmware_file);
 		return -1;
 	} 
 
 	return 0;
 }
 
-static int sm_drv_spi_down(struct net_device *dev)
+int sm_drv_spi_down(struct net_device *dev)
 {
-	struct net_local *lp = dev->priv;
-	struct spi_hif_local_data *hif_lp = HIF_LP(lp);
-	
 	DEBUG(DBG_CALLS, "%s\n", __FUNCTION__);
 
-	HIF_CLK_USE(hif_lp);
-
 	sm_drv_spi_reset_device(dev, 0);
 
-	HIF_CLK_UNUSE(hif_lp);
-
 	return 0;
 }
 
-static void sm_drv_spi_cli(struct net_device *dev)
+void sm_drv_spi_cli(struct net_device *dev)
 {
-	struct net_local *lp = dev->priv;
-	struct spi_hif_local_data *hif_lp = HIF_LP(lp);
-	unsigned int host_reg;
+	unsigned int host_reg = 0;
 	
-	HIF_CLK_USE(hif_lp);
-
-	host_reg = 0;
-	sm_spi_write(dev, SPI_ADRS_HOST_INT_EN, (unsigned char *)&host_reg, sizeof host_reg );
+	sm_spi_write(dev, SPI_ADRS_HOST_INT_EN,
+		     (unsigned char *) &host_reg, sizeof host_reg );
 	
 	cx3110x_disable_irq(dev);
-
-	HIF_CLK_UNUSE(hif_lp);
 }
 
 static void sm_drv_spi_initialize(struct net_device *dev)
@@ -760,15 +645,16 @@ static void sm_drv_spi_initialize(struct net_device *dev)
 	struct net_local *lp = dev->priv;
 	struct s_sm_setup setup;
 	unsigned long rand;
-	int i, callb_mask;
+	int i, callb_mask, ret;
 
 	DEBUG(DBG_CALLS, "%s\n", __FUNCTION__);
 
-	spin_lock_bh(&lp->sm_lock);
+	spin_lock_bh(&lp->lock);
 	
-	if(prism_softmac_describe(&lp->sm_descr, lp->sm_initdata) != SM_ENONE) {
-		printk(KERN_ERR "%s: Fatal Error: LMAC - UMAC incomptability\n", DRIVER_NAME );
-		return;
+	ret = prism_softmac_describe(&lp->sm_descr, lp->sm_initdata);
+	if (ret != SM_ENONE) {
+		cx3110x_error("LMAC - UMAC incomptability.");
+		goto out;
 	}
 
 	setup.mode = lp->sm_mode;
@@ -783,17 +669,19 @@ static void sm_drv_spi_initialize(struct net_device *dev)
 		setup.depth[i] = 0;
 
         /* allocate a memory region for the UMAC context */ 
-	lp->sm_context = (uint32_t *)kmalloc(lp->sm_descr.context_size, GFP_ATOMIC);
+	lp->sm_context = (uint32_t *)kmalloc(lp->sm_descr.context_size,
+					     GFP_ATOMIC);
 	if(!lp->sm_context) {
-		printk(KERN_ERR "%s: Fatal Error: UMAC Context Memory could not be allocated\n", DRIVER_NAME);
+		cx3110x_error("UMAC Context Memory could not be allocated.");
 		goto out;
 	}
 
 	DEBUG(DBG_CALLS, "sm_drv_spi_initialize: Mode %d\n", setup.mode);
 	
-	callb_mask = prism_softmac_create(lp->sm_context, &setup, lp->sm_pda, lp->sm_initdata);
+	callb_mask = prism_softmac_create(lp->sm_context, &setup, lp->sm_pda,
+					  lp->sm_initdata);
 	if(callb_mask < 0) {
-		printk(KERN_ERR "%s: Fatal Error: could not create SoftMAC (%d)\n", DRIVER_NAME, callb_mask);
+		cx3110x_error("Could not create SoftMAC (%d).", callb_mask);
 		lp->sm_descr.mtu = 0;
 	}
 	
@@ -803,17 +691,19 @@ static void sm_drv_spi_initialize(struct net_device *dev)
 	lp->sm_initialization = 1;
 
 out:
-	spin_unlock_bh(&lp->sm_lock);
+	spin_unlock_bh(&lp->lock);
 
+	/*
+	 * workaround for the set ladder problem
+	 * FIXME: fix this properly
+	 */
+	if (driver_type == SM_DRIVER_TYPE_MTUM) {
+		cx3110x_info("msleep(100)");
+		msleep(100);
+	}
 }
 
-#ifdef CONFIG_MACH_NOKIA770
-#define TARGET_BOOT_SLEEP 30
-#endif
-
-#ifdef CONFIG_MACH_NOKIA_N800
 #define TARGET_BOOT_SLEEP 50
-#endif
 
 static void sm_drv_spi_reset_device(struct net_device * dev, unsigned boot)
 {
@@ -845,7 +735,8 @@ static void sm_drv_spi_reset_device(struct net_device * dev, unsigned boot)
 	msleep(TARGET_BOOT_SLEEP);
 }
 
-int sm_drv_fetch_firmware(struct net_device *ndev, char *fw_id, struct device * dev)
+int sm_drv_fetch_firmware(struct net_device *ndev, char *fw_id,
+			  struct device * dev)
 {
 	struct net_local *lp = ndev->priv;
 	const struct firmware *fw_entry = 0;
@@ -855,24 +746,29 @@ int sm_drv_fetch_firmware(struct net_device *ndev, char *fw_id, struct device *
 	
 	ret = request_firmware(&fw_entry, fw_id, dev);
 	if (ret) {
-		printk(KERN_ERR "sm_drv_fetch_firmware failed (err: %d) for '%s' \n", ret, fw_id);
+		cx3110x_error("sm_drv_fetch_firmware failed for '%s': %d",
+			     fw_id, ret);
 		return ret;
 	}
+
+	spin_lock_bh(&lp->lock);
 	
 	lp->fw_len = fw_entry->size;
 	lp->fw_ptr = (uint8_t *)kmalloc(lp->fw_len, GFP_ATOMIC);
 	if (!lp->fw_ptr) {
-		printk("Couldn't allocate fw\n");
+		cx3110x_error("Couldn't allocate memory for firmware.");
 		ret = -1;
 		goto out;
 	}
 	
 	memcpy(lp->fw_ptr, fw_entry->data, lp->fw_len);
 
-	DEBUG(DBG_CALLS, "sm_drv_fetch_firmware: file %s (%d bytes)\n", fw_id, fw_entry->size);
+	DEBUG(DBG_CALLS, "sm_drv_fetch_firmware: file %s (%d bytes)\n",
+	      fw_id, fw_entry->size);
 
 	if (lp->fw_len % 4) {
-		printk(KERN_ERR "firmware '%s' size is not multiple of 32bit, aborting!\n", fw_id);
+		cx3110x_error("firmware '%s' size is not multiple of 32bit.",
+			      fw_id);
 		ret = -EILSEQ; /* Illegal byte sequence  */;
 		goto err_out;
 	}
@@ -880,18 +776,19 @@ int sm_drv_fetch_firmware(struct net_device *ndev, char *fw_id, struct device *
 	/* add offset 20 to point to the bra directly */ 
 	ret = prism_softmac_parse_bra(lp->fw_ptr + 20, lp->sm_initdata);
 	if(ret != SM_ENONE) {
-		printk(KERN_WARNING "%s firmware(%s) image does not contain LMAC description %d\n", DRIVER_NAME, fw_id, ret);
+		cx3110x_error("firmware does not contain LMAC description.");
 		ret = -1;
 		goto err_out;
 	}
 
  out:
+	spin_unlock_bh(&lp->lock);
+
 	/* Firmware version is at offset 40 */
 	memcpy(wlan_fw_version, fw_entry->data + 40, 16);
 	wlan_fw_version[16] = '\0';
 	
-	printk("%s: firmware version: %s\n",
-	       DRIVER_NAME, wlan_fw_version);
+	cx3110x_info("firmware version %s.", wlan_fw_version);
 
 	release_firmware(fw_entry);
 	return ret;
@@ -899,6 +796,9 @@ int sm_drv_fetch_firmware(struct net_device *ndev, char *fw_id, struct device *
  err_out:
 	kfree(lp->fw_ptr);
 	lp->fw_len = 0;
+
+	spin_unlock_bh(&lp->lock);
+
 	release_firmware(fw_entry);
 	return ret;	
 }
@@ -908,16 +808,19 @@ void sm_drv_release_firmware(struct net_device *ndev)
 {
 	struct net_local *lp = ndev->priv;
 
+	spin_lock_bh(&lp->lock);
+
 	if (lp->fw_ptr)
 		kfree(lp->fw_ptr);
 	lp->fw_len = 0;
+
+	spin_unlock_bh(&lp->lock);
 }
 
-
+/* FIXME: fw_len and fw_buffer needs to be locked */
 int sm_drv_spi_upload_firmware(struct net_device *dev)
 {
 	struct net_local *lp = dev->priv;
-	struct spi_hif_local_data *hif_lp = HIF_LP(lp);
 	struct s_dma_regs dma_regs;
 	unsigned long fw_len, address = FIRMWARE_ADDRESS;
 	uint32_t host_reg;
@@ -928,11 +831,10 @@ int sm_drv_spi_upload_firmware(struct net_device *dev)
 	fw_len = lp->fw_len;
 	fw_buffer = lp->fw_ptr;
 	
-	if (lp->fw_len == 0)
+	if (lp->fw_len == 0) {
 		return -1;
-
-	HIF_CLK_USE(hif_lp);
-
+	}
+	
 	/* Stop device */
 	sm_drv_spi_reset_device(dev, 0);
 
@@ -948,16 +850,19 @@ int sm_drv_spi_upload_firmware(struct net_device *dev)
 		fw_len -= _fw_len;
 		address += _fw_len;
 		
-		sm_spi_write(dev, SPI_ADRS_DMA_WRITE_CTRL, (unsigned char *)(&dma_regs.cmd), sizeof(short));
-		if (wlan_wait_bit(dev, SPI_ADRS_DMA_WRITE_CTRL, HOST_ALLOWED, HOST_ALLOWED) == 0) {
-			printk("fw_upload: Not allowed to DMA write\n");
+		sm_spi_write(dev, SPI_ADRS_DMA_WRITE_CTRL,
+			     (unsigned char *)(&dma_regs.cmd), sizeof(short));
+		if (wlan_wait_bit(dev, SPI_ADRS_DMA_WRITE_CTRL,
+				  HOST_ALLOWED, HOST_ALLOWED) == 0) {
+			cx3110x_warning("fw_upload not allowed to DMA write.");
 			cx3110x_dump_register(dev);
-			HIF_CLK_UNUSE(hif_lp);
 			return -EAGAIN; 
 		}
 		
-		sm_spi_write(dev, SPI_ADRS_DMA_WRITE_LEN, (unsigned char *)(&dma_regs.len), sizeof(short));
-		sm_spi_write(dev, SPI_ADRS_DMA_WRITE_BASE, (unsigned char *)(&dma_regs.addr), sizeof(long));
+		sm_spi_write(dev, SPI_ADRS_DMA_WRITE_LEN,
+			     (unsigned char *)(&dma_regs.len), sizeof(short));
+		sm_spi_write(dev, SPI_ADRS_DMA_WRITE_BASE,
+			     (unsigned char *)(&dma_regs.addr), sizeof(long));
 		
 		sm_spi_dma_write(dev, SPI_ADRS_DMA_DATA, fw_buffer, _fw_len);
 	}
@@ -966,13 +871,12 @@ int sm_drv_spi_upload_firmware(struct net_device *dev)
 
 	/* Enable host interrupts */
 	host_reg = SPI_HOST_INTS_DEFAULT;
-	sm_spi_write(dev, SPI_ADRS_HOST_INT_EN, (unsigned char *)&host_reg, sizeof host_reg );
+	sm_spi_write(dev, SPI_ADRS_HOST_INT_EN,
+		     (unsigned char *)&host_reg, sizeof host_reg );
 
 	/* Boot device */
 	sm_drv_spi_reset_device(dev, 1);
 
-	HIF_CLK_UNUSE(hif_lp);
-	
 	DEBUG(DBG_CALLS, "Upload firmware done\n");
 
 	return 0;
@@ -980,7 +884,8 @@ int sm_drv_spi_upload_firmware(struct net_device *dev)
 
 
 extern pda_record_t sm_drv_pda[];
-static struct wlan_pda_add_regulatory_limits_channel_s wlan_pda_add_regulatory_limits[4] =
+static struct wlan_pda_add_regulatory_limits_channel_s
+wlan_pda_add_regulatory_limits[4] =
 {
 	{
 		.frequency = 2412,
@@ -1034,9 +939,11 @@ struct s_pda * sm_drv_spi_get_pda(struct device *dev)
 				       wlan_pda_mac_address.mac_addr, 
 				       sizeof(wlan_pda_mac_address.mac_addr));
 	if (wlan_pda_iq)
-		sm_drv_pda_replace_pdr(sm_drv_pda, PDR_PRISM_ZIF_TX_IQ_CALIBRATION, 
+		sm_drv_pda_replace_pdr(sm_drv_pda,
+				       PDR_PRISM_ZIF_TX_IQ_CALIBRATION, 
 				       (uint8_t *)wlan_pda_iq,
-				       NUM_CHANNELS * sizeof(struct wlan_pda_iq_autocal_s));
+				       NUM_CHANNELS *
+				       sizeof(struct wlan_pda_iq_autocal_s));
 	
 	if (wlan_pda_pa_curve_data)
 		sm_drv_pda_replace_pdr(sm_drv_pda, PDR_PRISM_PA_CAL_CURVE_DATA,
@@ -1044,12 +951,14 @@ struct s_pda * sm_drv_spi_get_pda(struct device *dev)
 				       sizeof(struct wlan_pda_pa_curve_data_s));
 
 	if (wlan_pda_output_limits)
-		sm_drv_pda_replace_pdr(sm_drv_pda, PDR_PRISM_PA_CAL_OUTPUT_POWER_LIMITS,
+		sm_drv_pda_replace_pdr(sm_drv_pda,
+				       PDR_PRISM_PA_CAL_OUTPUT_POWER_LIMITS,
 				       (uint8_t *)wlan_pda_output_limits,
 				       sizeof(struct wlan_pda_output_limit_s));	
 
 	if (wlan_pda_rssi) {
-		sm_drv_pda_replace_pdr(sm_drv_pda, PDR_RSSI_LINEAR_APPROXIMATION_EXTENDED,
+		sm_drv_pda_replace_pdr(sm_drv_pda,
+				       PDR_RSSI_LINEAR_APPROXIMATION_EXTENDED,
 				       (uint8_t *)wlan_pda_rssi,
 				       sizeof(struct wlan_pda_rssi_cal_s));	
 	}
@@ -1070,44 +979,34 @@ struct s_pda * sm_drv_spi_get_pda(struct device *dev)
 }
 
 
-static irqreturn_t sm_drv_spi_interrupt(int irq, void *config, struct pt_regs *regs)
+static irqreturn_t sm_drv_spi_interrupt(int irq, void *config)
 {
 	struct net_device *dev = config;
 	struct net_local *lp = dev->priv;
 
-#ifdef CONFIG_MACH_NOKIA_N800
 	DEBUG(DBG_IRQ, "IRQ\n");
 
-	if(lp->device_state == DEVSTATE_ACTIVE || lp->device_state == DEVSTATE_BOOTING)
-		schedule_work(&lp->wq);
-#endif /* CONFIG_MACH_NOKIA_N800 */	
-	
-#ifdef CONFIG_MACH_NOKIA770
-	if(lp->device_state == DEVSTATE_ACTIVE || lp->device_state == DEVSTATE_BOOTING)
-	        tasklet_hi_schedule(&lp->tasklet);
-#endif /* CONFIG_MACH_NOKIA770 */
+	/* FIXME: move state check to workqueue */
+	if(lp->device_state == DEVSTATE_ACTIVE
+	   || lp->device_state == DEVSTATE_BOOTING)
+		queue_work(lp->workqueue, &lp->work);
 
 	return IRQ_HANDLED;
 }
 
 int sm_drv_spi_request_irq(struct net_device *dev)
 {
-	struct net_local *lp = dev->priv;
-	struct spi_hif_local_data *hif_lp = HIF_LP(lp);
 	int rval;
 	
 	DEBUG(DBG_CALLS, "%s\n", __FUNCTION__);
 	
-	HIF_CLK_USE(hif_lp);
-
 	/* Install interrupt handler */
-	rval = cx3110x_request_irq(sm_drv_spi_interrupt, DRIVER_NAME, (void*)dev);
+	rval = cx3110x_request_irq(sm_drv_spi_interrupt, DRIVER_NAME,
+				   (void*) dev);
 
 	if (rval < 0)
 		return rval;
        	
-	HIF_CLK_UNUSE(hif_lp);
-
 	return 0;
 }
 
@@ -1119,13 +1018,19 @@ void sm_drv_spi_free_irq( struct net_device *dev )
 
 struct sm_drv_firmware_file {
 	u8  chip_type;
-	u8* name; /* Chip's manufacturer name */
-	u8* lm;   /* LMAC name */
-	u8* mtlm; /* MTLM name */
+	/* Chip's manufacturer name */
+	u8* name; 
+	/* LMAC name */
+	u8* lm;
+	/* MTLM name */
+	u8* mtlm; 
 } firmware_array [] = {
-	{0x21, "STLC4370", "3825.arm", "mtlm3825.arm"}, /* STLC4370, aka gen2.1 */
-	{0x25, "STLC4550", "3826.arm", "mtlm3826.arm"}, /* STLC4550, aka gen2.5 */
-	{0x0,  "Unknown" , "3825.arm", "mtlm3825.arm"}, /* Default to STLC4370 */
+	/* STLC4370, aka gen2.1 */
+	{0x21, "STLC4370", "3825.arm", "mtlm3825.arm"}, 
+	/* STLC4550, aka gen2.5 */
+	{0x25, "STLC4550", "3826.arm", "mtlm3826.arm"}, 
+	/* Default to STLC4370 */
+	{0x0,  "Unknown" , "3825.arm", "mtlm3825.arm"}, 
 };
 
 static u8 * sm_drv_firmware_name(u8 chip_type, unsigned int driver_type)
@@ -1134,7 +1039,8 @@ static u8 * sm_drv_firmware_name(u8 chip_type, unsigned int driver_type)
 	
 	while (firmware_array[i].chip_type) {
 		if (firmware_array[i].chip_type == chip_type) {
-			printk("CX3110x chip variant: %s\n", firmware_array[i].name);
+			cx3110x_info("chip variant %s.",
+				     firmware_array[i].name);
 			if (driver_type == SM_DRIVER_TYPE_UMAC)
 				return firmware_array[i].lm;
 			else
@@ -1154,16 +1060,20 @@ static struct net_device *sm_drv_spi_probe(void)
 {
 	struct net_local *lp;
 	struct net_device * dev = NULL;
-	struct spi_hif_local_data *spi_lp = NULL;
+	struct spi_local *spi_lp = NULL;
 	const struct omap_wlan_cx3110x_config * cx3110x_wlan_config;
-
+	int ret;
+	
 	DEBUG(DBG_CALLS, "%s\n", __FUNCTION__);
 
-	/* Create the Network device instance that will interface with the kernel Network stack */
+	/*
+	 * Create the Network device instance that will interface with the
+	 * kernel Network stack
+	 */
 	dev = sm_drv_netdev_create(0,0);
 	
 	if (!dev) {
-		printk(KERN_ERR "Cannot allocate network device");
+		cx3110x_error("Cannot allocate network device.");
 		return NULL;
 	}
 	
@@ -1176,15 +1086,17 @@ static struct net_device *sm_drv_spi_probe(void)
 	if (driver_register(&wlan_omap_driver))
 		goto err_out_1;
 
-	sm_drv_sysfs_create_files();
+	ret = sm_drv_sysfs_create_files();
+	if (ret < 0)
+		goto err_out_2;
 	
 	lp = dev->priv;
-	spi_lp = HIF_LP(lp);
+	spi_lp = lp->spi_lp;
 	
 
-	spi_lp = (struct spi_hif_local_data *)kzalloc(sizeof(struct spi_hif_local_data), GFP_KERNEL);
+	spi_lp = kzalloc(sizeof(struct spi_local), GFP_KERNEL);
 	if (!spi_lp) {
-		printk(KERN_ERR "Cannot allocate SPI local data");
+		cx3110x_error("Cannot allocate SPI local data.");
 		goto err_out_2;
 	}
 	
@@ -1197,7 +1109,7 @@ static struct net_device *sm_drv_spi_probe(void)
 		return NULL;
 	
 	cx3110x_wlan_config = omap_get_config(OMAP_TAG_WLAN_CX3110X, 
-					      struct omap_wlan_cx3110x_config);	
+					      struct omap_wlan_cx3110x_config);
 	
 	/* With some old N770 bootloaders, the chip type s not set */
 	if (cx3110x_wlan_config == NULL ||
@@ -1208,39 +1120,23 @@ static struct net_device *sm_drv_spi_probe(void)
 		wlan_config->irq_gpio = N770_WLAN_IRQ_GPIO;
 		wlan_config->spi_cs_gpio = N770_WLAN_CS_GPIO;
 	} else {
-		memcpy(wlan_config, cx3110x_wlan_config, sizeof(struct omap_wlan_cx3110x_config));
+		memcpy(wlan_config, cx3110x_wlan_config,
+		       sizeof(struct omap_wlan_cx3110x_config));
 	}
 
 
-#ifdef CONFIG_MACH_NOKIA770
-	/* McBSP2 clock is ARM_PER */
-	HIF_CLK(spi_lp) = clk_get(NULL, "armper_ck");
-	BUG_ON(HIF_CLK(spi_lp) == NULL);
-#endif /* CONFIG_MACH_NOKIA770 */
-
-	/* Instantiate the Host Interface specific handlers */
-	lp->hif_up      = &sm_drv_spi_up;
-	lp->hif_down    = &sm_drv_spi_down;
-	lp->hif_cli     = &sm_drv_spi_cli;
-	lp->hif_local_data = spi_lp;            /* point back to SPI local data */
+	lp->spi_lp = spi_lp;
 
 	spin_lock_init(&spi_lp->lock);
 	
 	/* Initialise initial packet count */
-	spi_lp->initial_packets = 0;
 	spi_lp->spi_packets = 0;
 
 	/* Init our Bottom halve handler */
-#ifdef CONFIG_MACH_NOKIA_N800
-	INIT_WORK(&lp->wq, sm_drv_spi_wq, dev);
-#endif /* CONFIG_MACH_NOKIA_N800 */
-
-#ifdef CONFIG_MACH_NOKIA770
-	tasklet_init(&lp->tasklet, sm_drv_spi_tasklet_action, (unsigned long)dev);
-#endif /* CONFIG_MACH_NOKIA770 */
+	lp->workqueue = create_singlethread_workqueue("cx3110x");
+	INIT_WORK(&lp->work, sm_drv_spi_wq);
 
 	/* Init the wireless stats queue */
-	INIT_WORK(&lp->stats_work, sm_drv_update_stats, dev);
 	lp->stats_timestamp = 0;
 
 	/* Init the WPA IE list */
@@ -1266,20 +1162,24 @@ static struct net_device *sm_drv_spi_probe(void)
 	/* Default country is EU if this is NULL */
 	wlan_pda_default_country = NULL;
 
-	lp->sm_initdata = (struct s_sm_initdata *) kmalloc(sizeof(struct s_sm_initdata), GFP_KERNEL );
+	lp->sm_initdata = kmalloc(sizeof(struct s_sm_initdata), GFP_KERNEL);
 	if(!lp->sm_initdata) {
-		printk(KERN_ERR "Cannot allocate SPI initdata");
+		cx3110x_error("Cannot allocate SPI initdata.");
 		goto err_out_4;
 	}
 
+	SET_NETDEV_DEV(dev, &wlan_omap_device.dev);
+
 	register_netdev(dev);
 	
 	/* Let's fetch the firmware from userspace */
-	firmware_file = sm_drv_firmware_name(wlan_config->chip_type, driver_type);
+	firmware_file = sm_drv_firmware_name(wlan_config->chip_type,
+					     driver_type);
 	if (firmware_file == NULL)
 		goto err_out_4;
 	
-	if (sm_drv_fetch_firmware(dev, firmware_file, &(wlan_omap_device.dev)) < 0)
+	if (sm_drv_fetch_firmware(dev, firmware_file,
+				  &(wlan_omap_device.dev)) < 0)
 		goto err_out_4;
 		
 	return dev;
@@ -1305,10 +1205,6 @@ static void sm_drv_spi_unprobe(struct net_device *dev)
 	kfree(wlan_config);
 }
 
-/******************************************************************************
- *   Module Init
- ******************************************************************************/
-#ifdef CONFIG_MACH_NOKIA_N800
 static struct spi_driver cx3110x_driver = {
 	.driver = {
 		.name		= "cx3110x",
@@ -1319,42 +1215,36 @@ static struct spi_driver cx3110x_driver = {
 	.probe		= cx3110x_probe,
 	.remove		= __devexit_p(cx3110x_remove),
 };
-#endif /* CONFIG_MACH_NOKIA_N800 */
 
 static int __init sm_drv_spi_init_module(void)
 {
-#ifdef CONFIG_MACH_NOKIA_N800
 	int status;
-#endif /* CONFIG_MACH_NOKIA_N800 */
 
 	sm_net_device = sm_drv_spi_probe();
 
 	if (!sm_net_device) {
-		printk(KERN_WARNING "Could not register SoftMAC SPI Driver\n");
+		cx3110x_error("Could not register SoftMAC SPI Driver.");
 		return -ENODEV;
 	}
 
-#ifdef CONFIG_MACH_NOKIA_N800
 	status = spi_register_driver(&cx3110x_driver);
 	if (status < 0)
 		return status;
-#endif /* CONFIG_MACH_NOKIA_N800 */
 	
-	printk("Loaded %s driver, version %s\n", DRIVER_NAME, VERSIONID);
+	cx3110x_info("driver version %s loaded.", DRIVER_VERSION);
 
 	return 0;
 }
 
 static void __exit sm_drv_spi_cleanup_module( void )
 {
-#ifdef CONFIG_MACH_NOKIA_N800
 	spi_unregister_driver(&cx3110x_driver);
-#endif /* CONFIG_MACH_NOKIA_N800 */
 	sm_drv_spi_unprobe(sm_net_device);
 
-	printk("Unloaded %s  driver, version %s\n", DRIVER_NAME, VERSIONID);
+	cx3110x_info("driver version %s unloaded.", DRIVER_VERSION);
 }
-
+MODULE_DESCRIPTION("WLAN driver for Nokia N800");
+MODULE_AUTHOR("Kalle Valo <Kalle.Valo@nokia.com>");
 MODULE_LICENSE("GPL v2");
 
 module_init(sm_drv_spi_init_module);
